#include <math.h>
#include <stdlib.h>

double lambda( double x )
{
	if ( x <= 2.50 )
	{
		return 0.737646 - 0.447371 * x + 0.474084 * pow( x, 2 )  - 0.164706 * pow( x, 3 )  + 0.0253832 * pow( x, 4 );
	}
	else
	{
		return -9.25522 + 11.8917 * x - 4.88355 * pow( x, 2 )  + 0.707071 * pow( x, 3 );
	}
}

double afr( double l )
{
	return 14.70 * l;
}

int main( int argc, char *argv[] )
{
	int x, l;
	double v;
	char a[ 10 ], last_a[ 10 ];

	if ( argc >= 2 && argv[ 1 ][ 0 ] == 'l' )	l = 1;
	else										l = 0;
	
	a[ 0 ]		= '\0';
	last_a[ 0 ]	= '\0';

	printf( "#ifndef __afr_h\n" );
	printf( "#define __afr_h\n" );
	printf( "\n" );
	printf( "typedef struct _AFRLookup\n" );
	printf( "{\n" );
	printf( "	int		Vout;\n" );
	printf( "	double	AFR;\n" );
	printf( "} AFRLookup;\n" );
	printf( "\n" );
	printf( "static AFRLookup %s_lookup_table[] =\n", l ? "lambda" : "afr" );
	printf( "{\n" );

	x = 0;
	for ( v = 1.400; v <= 2.900; v += 0.001 )
	{
		if ( l )	sprintf( a, "%.02f", lambda( v ) );
		else		sprintf( a, "%.01f", afr( lambda( v ) ) );
		if ( strcmp( a, last_a ) )
		{
			if ( l )	printf( "	{ %d, %.02f },", ( int ) ( v * 1000 ), lambda( v ) );
			else		printf( "	{ %d, %.01f },", ( int ) ( v * 1000 ), afr( lambda( v ) ) );
			if ( ( ++x % 5 ) == 0 )
			{
				printf( "\n" );
			}
			strcpy( last_a, a );
		}
	}

	printf( "\n" );
	printf( "};\n" );
	printf( "\n" );
	printf( "#define %s_TABLE_SIZE	%d\n", l ? "LAMBDA" : "AFR", x );
	printf( "\n" );
	printf( "#endif\n" );

	return EXIT_SUCCESS;
}
